@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -533,6 +533,18 @@ public class Get extends Task {
         extends org.apache.tools.ant.util.Base64Converter {
     }
 
+    /**
+     * Does the response code represent a redirection?
+     *
+     * @since 1.10.10
+     */
+    public static boolean isMoved(final int responseCode) {
+        return responseCode == HttpURLConnection.HTTP_MOVED_PERM
+            || responseCode == HttpURLConnection.HTTP_MOVED_TEMP
+            || responseCode == HttpURLConnection.HTTP_SEE_OTHER
+            || responseCode == HTTP_MOVED_TEMP;
+    }
+
     /**
      * Interface implemented for reporting
      * progress of downloading.
@@ -693,29 +705,34 @@ public class Get extends Task {
 
 
         private boolean redirectionAllowed(final URL aSource, final URL aDest) {
-            if (aSource.getProtocol().equals(aDest.getProtocol())
-                    && (HTTP.equals(aSource.getProtocol()) || HTTPS.equals(aDest.getProtocol()))) {
-                redirections++;
-                if (redirections > REDIRECT_LIMIT) {
-                    final String message = "More than " + REDIRECT_LIMIT
-                            + " times redirected, giving up";
-                    if (ignoreErrors) {
-                        log(message, logLevel);
-                        return false;
-                    }
+            if (!(aSource.getProtocol().equals(aDest.getProtocol()) || (HTTP
+                    .equals(aSource.getProtocol()) && HTTPS.equals(aDest
+                    .getProtocol())))) {
+                final String message = "Redirection detected from "
+                        + aSource.getProtocol() + " to " + aDest.getProtocol()
+                        + ". Protocol switch unsafe, not allowed.";
+                if (ignoreErrors) {
+                    log(message, logLevel);
+                    return false;
+                } else {
                     throw new BuildException(message);
                 }
-                return true;
             }
 
-            final String message = "Redirection detected from "
-                    + aSource.getProtocol() + " to " + aDest.getProtocol()
-                    + ". Protocol switch unsafe, not allowed.";
-            if (ignoreErrors) {
-                log(message, logLevel);
-                return false;
+            redirections++;
+            if (redirections > REDIRECT_LIMIT) {
+                final String message = "More than " + REDIRECT_LIMIT
+                        + " times redirected, giving up";
+                if (ignoreErrors) {
+                    log(message, logLevel);
+                    return false;
+                } else {
+                    throw new BuildException(message);
+                }
             }
-            throw new BuildException(message);
+
+
+            return true;
         }
 
         private URLConnection openConnection(final URL aSource) throws IOException {
@@ -810,13 +827,6 @@ public class Get extends Task {
             return connection;
         }
 
-        private boolean isMoved(final int responseCode) {
-            return responseCode == HttpURLConnection.HTTP_MOVED_PERM
-                    || responseCode == HttpURLConnection.HTTP_MOVED_TEMP
-                    || responseCode == HttpURLConnection.HTTP_SEE_OTHER
-                    || responseCode == HTTP_MOVED_TEMP;
-        }
-
         private boolean downloadFile() throws IOException {
             for (int i = 0; i < numberRetries; i++) {
                 // this three attempt trick is to get round quirks in different
